# make-art.py
import board, time, digitalio, neopixel, touchio
from audiomp3 import MP3Decoder

try:
    from audioio import AudioOut
except ImportError:
    try:
        from audiopwmio import PWMAudioOut as AudioOut
    except ImportError:
        print("This board does not support audio")

# speaker
speaker = digitalio.DigitalInOut(board.SPEAKER_ENABLE)
speaker.direction = digitalio.Direction.OUTPUT
speaker.value = True

# audio + decoder
#note MAKE SURE TO CHANGE FILENAME FOR DECODER IF YOU ARE USING DIFFERENT SONGS!
audio = AudioOut(board.SPEAKER)
path = "sounds/"
mp3_file = open(path + "sweet_escape_easy.mp3", "rb")
decoder = MP3Decoder(mp3_file)

#neopixel + colors
pixels = neopixel.NeoPixel(board.A3, 4, brightness=0.5)

BLACK = (0, 0, 0)

YELLOW = (255, 150, 0)
RED = (255, 0, 0)
GREEN = (0,255,0)
BLUE = (0, 0, 255)

PINK = (242, 90, 255)
CYAN = (0, 255, 255)
WHITE = (255, 255, 255)
JADE = (0, 255, 40)

pixels.fill(BLACK)

#CONSTANTS
SEC = 1_000_000_000
UP_BUTTON = touchio.TouchIn(board.A4)
RIGHT_BUTTON = touchio.TouchIn(board.A5)
DOWN_BUTTON = touchio.TouchIn(board.A6)
LEFT_BUTTON = touchio.TouchIn(board.TX)

BUTTONS = [UP_BUTTON, RIGHT_BUTTON, DOWN_BUTTON, LEFT_BUTTON]

UP_LIGHT = 1
RIGHT_LIGHT = 2
DOWN_LIGHT = 3
LEFT_LIGHT = 0

LIGHTS = [UP_LIGHT, RIGHT_LIGHT, DOWN_LIGHT, LEFT_LIGHT]

#mappings
#each dictionary has key timestamps,
#and value params for dir to perform
#0 is up, 1 is right, 2 is down, 3 is left
sweet_times = {
    9.1: (3, BLUE, 1 * SEC),
    11.1: (1, RED, 1 * SEC),
    13.1: (3, BLUE, 1 * SEC),
    15.1: (1, RED, 1 * SEC),
    17.1: (3, BLUE, 1 * SEC),
    19.1: (1, RED, 1 * SEC),
    21.1: (0, YELLOW, 1 * SEC),
    23.1: (2, GREEN, 1 * SEC),
    25.1: (0, YELLOW, 1 * SEC),
    27.1: (2, GREEN, 1 * SEC),
    29.1: (3, BLUE, 1 * SEC),
    31.1: (1, RED, 1 * SEC),
    33.1: (0, YELLOW, 1 * SEC),
    35.1: (2, GREEN, 1 * SEC),
    37.1: (3, BLUE, 1 * SEC),
    39.1: (1, RED, 1 * SEC),
    41.1: (0, YELLOW, 0.5 * SEC),
    42.1: (1, RED, 0.5 * SEC),
    43.1: (0, YELLOW, 0.5 * SEC),
    44.1: (1, RED, 0.5 * SEC),
    45.1: (2, GREEN, 0.5 * SEC),
    46.1: (3, BLUE, 0.5 * SEC),
    47.1: (2, GREEN, 0.5 * SEC),
    51.2: (0, YELLOW, 0.4 * SEC),
    51.7: (1, RED, 0.4 * SEC),
    52.2: (2, GREEN, 0.4 * SEC),
    52.7: (3, BLUE, 0.4 * SEC),
    53.1: (0, YELLOW, 0.4 * SEC),
    55.2: (3, BLUE, 0.5 * SEC),
    56.2: (3, BLUE, 0.5 * SEC),
    57.2: (1, RED, 0.5 * SEC),
    58.2: (1, RED, 0.5 * SEC),
    59.2: (3, BLUE, 0.5 * SEC),
    60.2: (3, BLUE, 0.5 * SEC),
    61.2: (1, RED, 0.5 * SEC),
    62.2: (1, RED, 0.5 * SEC),
    63.2: (3, BLUE, 0.5 * SEC),
    64.2: (3, BLUE, 0.5 * SEC),
    67.1: (2, BLUE, 0.4 * SEC),
    67.6: (1, GREEN, 0.4 * SEC),
    68.1: (0, RED, 0.4 * SEC),
    68.6: (3, YELLOW, 0.4 * SEC),
    69.1: (2, BLUE, 0.4 * SEC),
    71.2: (0, YELLOW, 0.5 * SEC),
    72.1: (0, YELLOW, 0.5 * SEC),
    73.1: (2, GREEN, 0.5 * SEC),
    74.1: (2, GREEN, 0.5 * SEC),
    75.1: (0, YELLOW, 0.5 * SEC),
    76.1: (0, YELLOW, 0.5 * SEC),
    77.1: (2, GREEN, 0.5 * SEC),
    78.1: (2, GREEN, 0.5 * SEC),
    79.1: (0, YELLOW, 0.5 * SEC),
    80.1: (0, YELLOW, 0.5 * SEC),
    82.0: (3, BLUE, 1 * SEC),
    84.0: (1, RED, 1 * SEC),
    86.0: (3, BLUE, 1 * SEC),
    88.0: (1, RED, 1 * SEC),
    90.0: (0, YELLOW, 1 * SEC),
    92.0: (2, GREEN, 1 * SEC),
    94.0: (0, YELLOW, 1 * SEC),
    96.0: (2, GREEN, 1 * SEC)
}

american_times = {
    8.6: ((1, 3), JADE, 0.9 * SEC),
    10.7: ((0, 2), JADE, 0.9 * SEC),
    12.7: ((0, 1), PINK, 0.9 * SEC),
    15.8: ((2, 3), PINK, 0.9 * SEC),
    16.8: ((1, 3), JADE, 0.9 * SEC),
    18.8: ((0, 2), JADE, 0.9 * SEC),
    20.8: ((1, 3), JADE, 0.9 * SEC),
    25.0: ((1, 3), JADE, 0.4 * SEC),
    25.4: ((1, 3), JADE, 0.4 * SEC),
    25.9: ((1, 3), JADE, 0.4 * SEC),
    27.4: ((1, 3), JADE, 0.4 * SEC),
    27.9: ((1, 3), JADE, 0.4 * SEC),
    29.0: ((0, 2), JADE, 0.4 * SEC),
    29.4: ((0, 2), JADE, 0.4 * SEC),
    30.0: ((0, 2), JADE, 0.9 * SEC),
    31.5: ((0, 2), JADE, 0.4 * SEC),
    32.0: ((0, 2), JADE, 0.9 * SEC),
    33.1: ((1, 3), JADE, 0.4 * SEC),
    33.5: ((1, 3), JADE, 0.4 * SEC),
    34.0: ((1, 3), JADE, 0.9 * SEC),
    35.6: ((1, 3), JADE, 0.4 * SEC),
    36.1: ((1, 3), JADE, 0.4 * SEC),
    37.1: ((0, 2), JADE, 0.4 * SEC),
    37.6: ((0, 2), JADE, 0.9 * SEC),
    38.1: ((0, 2), JADE, 0.9 * SEC),
    41.2: (1, WHITE, 0.4  * SEC),
    41.7: (3, WHITE, 0.4  * SEC),
    42.2: (1, WHITE, 0.4  * SEC),
    42.7: (3, WHITE, 0.4  * SEC),
    43.2: (2, WHITE, 0.4  * SEC),
    43.7: (0, WHITE, 0.4  * SEC),
    44.2: (2, WHITE, 0.4  * SEC),
    44.7: (0, WHITE, 0.4  * SEC),
    45.3: (3, WHITE, 0.4  * SEC),
    45.7: (1, WHITE, 0.4  * SEC),
    46.3: (3, WHITE, 0.4  * SEC),
    46.8: (1, WHITE, 0.4  * SEC),
    47.3: (0, WHITE, 0.4 * SEC),
    47.8: (2, WHITE, 0.4 * SEC),
    48.3: (0, WHITE, 0.4 * SEC),
    48.8: (2, WHITE, 0.4 * SEC),
    49.8: ((0, 3), CYAN, 0.9 * SEC),
    50.8: ((0, 3), CYAN, 0.9 * SEC),
    51.8: ((0, 1), PINK, 0.9 * SEC),
    52.9: ((0, 1), PINK, 0.9 * SEC),
    53.9: ((1, 2), CYAN, 0.9 * SEC),
    54.8: ((1, 2), CYAN, 0.4 * SEC),
    54.9: ((1, 2), CYAN, 0.9 * SEC),
    55.9: ((2, 3), PINK, 0.9 * SEC),
    56.9: ((2, 3), PINK, 0.4 * SEC),
    57.5: ((1, 3), JADE, 0.9 * SEC),
    59.5: ((0, 2), JADE, 0.9 * SEC),
    61.5: ((0, 3), CYAN, 0.9 * SEC),
    64.5: ((1, 2), CYAN, 0.9 * SEC),
    65.6: ((1, 3), JADE, 0.9 * SEC),
    67.6: ((0, 2), JADE, 0.9 * SEC),
    73.7: ((0, 3), CYAN, 0.4 * SEC),
    74.2: ((1, 2), CYAN, 0.4 * SEC),
    74.7: ((0, 3), CYAN, 0.4 * SEC),
    76.2: ((0, 3), CYAN, 0.4 * SEC),
    76.7: ((1, 2), CYAN, 0.4 * SEC),
    77.8: ((0, 1), PINK, 0.4 * SEC),
    78.3: ((2, 3), PINK, 0.4 * SEC),
    78.8: ((0, 1), PINK, 0.4 * SEC),
    80.3: ((0, 1), PINK, 0.4 * SEC),
    80.8: ((2, 3), PINK, 0.4 * SEC),
    81.8: ((1, 3), JADE, 0.4 * SEC),
    82.3: ((1, 3), JADE, 0.4 * SEC),
    82.9: ((1, 3), JADE, 0.4 * SEC),
    84.4: ((1, 3), JADE, 0.4 * SEC),
    84.9: ((1, 3), JADE, 0.9 * SEC),
    85.9: ((0, 2), JADE, 0.9 * SEC),
    86.4: ((0, 2), JADE, 0.4 * SEC),
    86.9: ((0, 2), JADE, 0.9 * SEC)
}

def test_mp3(filename):
    decoder.file = open(path + filename, "rb")
    audio.play(decoder)
    start = time.monotonic_ns()
    while audio.playing:
        time.sleep(0.05)
        curr = round((time.monotonic_ns() - start) / SEC, 1)
        for i, pad in enumerate(BUTTONS):
            if pad.value:
                print(f"{i} touched at {curr}")

def play_mp3(filename):
    decoder.file = open(path + filename, "rb")
    audio.play(decoder)
    while audio.playing:
        pass

def play_game(filename, times):
    decoder.file = open(path + filename, "rb")
    audio.play(decoder)
    start = time.monotonic_ns()
    while audio.playing:
        time.sleep(0.05)
        curr = round((time.monotonic_ns() - start) / SEC, 1)
        if curr in times:
            success = light_dir(*times[curr], time.monotonic_ns())
            if not success:
                audio.stop()
                play_mp3("lose.mp3")
                time.sleep(1)
                return
    play_mp3("win.mp3")
    time.sleep(1)

def light_dir(dir, color, window, start):
    multi = (type(dir) == tuple)
    curr = time.monotonic_ns()
    if not multi:
        button = BUTTONS[dir]
        light = LIGHTS[dir]
        pixels[light] = color
    else:
        light1 = LIGHTS[dir[0]]
        light2 = LIGHTS[dir[1]]
        button1 = BUTTONS[dir[0]]
        button2 = BUTTONS[dir[1]]
        pixels[light1] = color
        pixels[light2] = color

    while curr - start < window:
        time.sleep(0.02)
        curr = time.monotonic_ns()
        if not multi:
            if button.value:
                pixels.fill(BLACK)
                return True
        else:
            if button1.value and button2.value:
                pixels.fill(BLACK)
                return True
    pixels.fill(BLACK)
    return False

TEST = "american.mp3"

while True:
    if UP_BUTTON.value:
        play_mp3("intro.mp3")
    elif RIGHT_BUTTON.value:
        play_game("american.mp3", american_times)
    elif LEFT_BUTTON.value:
        play_game("sweet_escape_easy.mp3", sweet_times)
    elif DOWN_BUTTON.value:
        test_mp3(TEST)